#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "applyresult4xi.py">
#<abstract4xi abstract = "provides class ApplyResult4Xi" />
#<refer4xi reference = "pool.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------
#
# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'ApplyResult4Xi' ]
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass    
try:                 # most-common ones
    import itertools
    import time         
except:
    pass
try:                 # rare ones
    import queue 
    import collections           
except:
    pass 
       
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass

#"
#/>
 
# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from constant_pool4xi import next_job_counter_pool4xi
import constant_synchronize4xi
import constant_system4xi
from loop4xi import limit2loop4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from sleep4xi import omega2sleep4xi
from timeout4xi import omega2timeout4xi    
from timeouterror4xi import TimeoutError4Xi
from print4xi import print4xi
from sleep4xi import omega2sleep4xi
from timeout4xi import omega2timeout4xi
from time4xi import time4xi
#"
#/>

#
# =================================================================================================
#<class4xi class = "ApplyResult4Xi" base = "object" child = "None" synopsis = "Used for pooling results" usage = "
#   pool.py
#       instances are returned by Pool.apply_async()                             
#   apply_async funtion in pool.py:
#       result = ApplyResult(self._cache, callback)
#
#   inherited classes:
#       class MapResult(ApplyResult):
#
#   mapresult4xi.py
#       MapResult4Xi(ApplyResult4Xi)
#           ApplyResult4Xi.__init__(self, cache, callback)
#
#   pool4xi.py:
#       result2xi = MapResult4Xi(self._cache, chunksize, len(iterable), callback)
#"
#> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
#
class ApplyResult4Xi(object):
    '''
    used by pool4xi.py as follows: result2xi = ApplyResult4Xi(self._cache, callback)  
    '''
    
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi exposure = "
    _exposed_ = ('ready', 'successful', 'wait', 'get', 'sense', 'reason', 'ace')       
    #"
    #/>

        
    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    ack4manoeuvres = 0      # number of successful manoeuvres
    nack4manoeuvres = 0     # number of failed manoeuvres
    #"
    #/>
                                                                       
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-ApplyResult4Xi" usage = "initiliazes ApplyResult4Xi">
    # ---------------------------------------------------------------------------------------------
    def __init__(self, cache, callback, error_callback):
        '''
        initiliazes ApplyResult4Xi 
        '''

        # -----------------------------------------------------------------------------------------
        # (Step 1/8)           
        # -----------------------------------------------------------------------------------------
        class2print = 'ApplyResult4Xi'
        method2print = '__init__'
        
    
        message2print = "{(Step 1/8a) started" + "; init-of-ApplyResult4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #PX: 04:49:12 04/20/10 Pacific Daylight Time
                                
        
        message2print = "{(Step 1/8b) borning" + "; applyresult2xi-of-ApplyResult4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        #PX: 04:49:12 04/20/10 Pacific Daylight Time
                  
        message2print = "{(Step 1/8c) inheriting" + "; object-inheriting-by-applyresult2xi-of-ApplyResult4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #PX: <class 'object'>
              
        message2print = "{(Step 1/8d) passed" + "; cache-passed-to-init-of-ApplyResult4Xi: " + str(cache) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #PX: {}
              
        message2print = "{(Step 1/8e) passed" + "; callback-passed-to-init-of-ApplyResult4Xi: " + str(callback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #PX: None
        
        message2print = "{(Step 1/8f) passed" + "; error-callback-passed-to-init-of-ApplyResult4Xi: " + str(error_callback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #
                                
        # -----------------------------------------------------------------------------------------    
        # (Step 2/8)
        # -----------------------------------------------------------------------------------------        
        self._event = threading.Event()
            
        message2print = "{(Step 2/8a) set" + "; event-of-applyresult2xi-set-by-init-of-ApplyResult4Xi to: " + str(self._event)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #
        
        ApplyResult4Xi.ack4manoeuvres = ApplyResult4Xi.ack4manoeuvres + 1 
                  
          
        # -----------------------------------------------------------------------------------------    
        # (Step 3/8)
        # -----------------------------------------------------------------------------------------        
        self._job = next_job_counter_pool4xi()                                          #dragonized 
            
        message2print = "{(Step 3/8a) set" + "; job-of-applyresult2xi-set-by-init-of-ApplyResult4Xi to: " + str(self._job)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #
                  
        ApplyResult4Xi.ack4manoeuvres = ApplyResult4Xi.ack4manoeuvres + 1          
        
        # -----------------------------------------------------------------------------------------    
        # (Step 4/8)
        # -----------------------------------------------------------------------------------------        
        self._cache = cache
            
        message2print = "{(Step 4/8a) set" + "; cache-of-applyresult2xi-set-by-init-of-ApplyResult4Xi to: " + str(self._cache)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                               
        ApplyResult4Xi.ack4manoeuvres = ApplyResult4Xi.ack4manoeuvres + 1             
                          
        # -----------------------------------------------------------------------------------------    
        # (Step 5/8)
        # -----------------------------------------------------------------------------------------        
        self._callback = callback
        message2print = "{(Step 5/8a) set" + "; callback-of-applyresult2xi-set-by-init-of-ApplyResult4Xi to: " + str(self._callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
                           
        ApplyResult4Xi.ack4manoeuvres = ApplyResult4Xi.ack4manoeuvres + 1        
                  
        # -----------------------------------------------------------------------------------------    
        # (Step 6/8)
        # -----------------------------------------------------------------------------------------        
        self._error_callback = error_callback
        message2print = "{(Step 6/8a) set" + "; error_callback-of-applyresult2xi-set-by-init-of-ApplyResult4Xi to: " + str(self._error_callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #
                           
            
        # -----------------------------------------------------------------------------------------    
        # (Step 7/8)
        # -----------------------------------------------------------------------------------------        
        cache[self._job] = self
            
        message2print = "{(Step 7/8a1) using" + "; job-of-applyresult2xi-using-by-init-of-ApplyResult4Xi: " + str(self._job)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)        
        #PX: 0
            
        message2print = "{(Step 7/8a2) set" + "; cache-job-set-by-init-of-ApplyResult4Xi to: " + str(cache[self._job])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #PX: <applyresult4xi.ApplyResult4Xi object at 0x035F8F90>

     
        message2print = "{(Step 7/8a3) analyzing" + "; cache-analyzing-by-init-of-ApplyResult4Xi: " + str(cache) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                
        #PX: {0: <applyresult4xi.ApplyResult4Xi object at 0x030E04D0>}
            
        ApplyResult4Xi.ack4manoeuvres = ApplyResult4Xi.ack4manoeuvres + 1 
              

                                                                       
        # -----------------------------------------------------------------------------------------    
        # (Step 8/8)
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 8/8a) inherited" + "; object-inherited-by-applyresult2xi-of-ApplyResult4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #PX: <class 'object'>
               
        message2print = "{(Step 8/8b) activated" + "; applyresult2xi-of-ApplyResult4Xi-activated: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        #PX: 04:49:12 04/20/10 Pacific Daylight Time
      
        message2print = "{(Step 8/8c) calculated" + "; ack4manoeuvres-calculated-by-init-of-ApplyResult4Xi: " + str(ApplyResult4Xi.ack4manoeuvres) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        #PX: 6
      
        message2print = "{(Step 8/8d) calculated" + "; nack4manoeuvres-calculated-by-init-of-ApplyResult4Xi: " + str(ApplyResult4Xi.nack4manoeuvres) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        #PX: 0
                                  

        message2print = "{(Step 8/8e) returning" + "; init-of-ApplyResult4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #PX: 04:49:08 04/20/10 Pacific Daylight Time
                           
    #</methodize4xi> 
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ready-of-ApplyResult4Xi" usage = "returns ready">
    # ---------------------------------------------------------------------------------------------
    def ready(self):
        '''
        returns ready 
        '''
        class2print = 'ApplyResult4Xi'
        method2print = 'ready'        
    
        message2print = "{started" + "; ready-of-ApplyResult4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------        

        message2print = "{returning" + "; ready-of-applyresult2xi-returning-by-ready-of-ApplyResult4Xi: " + str(self._event.is_set) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)           
        # 
        
        return self._event.is_set()
        
    #</methodize4xi> 
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "successful-of-ApplyResult4Xi" usage = "return success">
    # ---------------------------------------------------------------------------------------------
    def successful(self):
        '''
        return success 
        '''
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------          
        class2print = 'ApplyResult4Xi'
        method2print = 'successful'        
    
           
        message2print = "{started" + "; successful-of-ApplyResult4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        
        assert self.ready()
        
        return self._success
            
    #</methodize4xi> 
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "wait-of-ApplyResult4Xi" usage = "waits with defined timeout">
    # ---------------------------------------------------------------------------------------------
    def wait(self, timeout=None):
        '''
        waits defined timeout
        ''' 
                   
        # -----------------------------------------------------------------------------------------    
        # (Step 1/4) 
        # -----------------------------------------------------------------------------------------        
                
        class2print = 'ApplyResult4Xi' 
        method2print = 'wait'        
        
        message2print = "{(Step 1/4a) started" + "; wait-of-ApplyResult4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                 

        timeout = omega2timeout4xi(class2print, method2print) #dragonized timeout
    
        message2print = "{(Step 1/4b) set" + "; timeout-set-by-wait-of-ApplyResult4Xi to: " + str(timeout)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        self._event.wait(timeout)
        
        # -----------------------------------------------------------------------------------------    
        # (Step 4/4) 
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 4/4) returning" + "; wait-of-ApplyResult4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      
        # 
        
    #</methodize4xi> 
    

                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get-of-ApplyResult4Xi" usage = "gets with defined timeout">
    # ---------------------------------------------------------------------------------------------
    def get(self, timeout=None):
        '''
        gets with defined timeout
        '''
        # -----------------------------------------------------------------------------------------    
        # (Step 1/6) 
        # -----------------------------------------------------------------------------------------              
        class2print = 'ApplyResult4Xi'
        method2print = 'get'        
    
        
        message2print = "{(Step 1/6) started" + "; get-of-ApplyResult4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #PX: 06:59:29 04/20/10 Pacific Daylight Time 
                 
        # -----------------------------------------------------------------------------------------    
        # (Step 2/6) 
        # -----------------------------------------------------------------------------------------       
        timeout = omega2timeout4xi(class2print, method2print)        #dragonized
        
        message2print = "{(Step 2/6) set" + "; timeout-set-by-get-of-ApplyResult4Xi to: " + str(timeout)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #PX: 0.2
                 
        # -----------------------------------------------------------------------------------------    
        # (Step 3/6) 
        # -----------------------------------------------------------------------------------------                  
        ack2wait = False
        nack2wait = True
        
        try:
            self.wait(timeout)
        
            ack2wait = True
            
            message2print = "{(Step 3/6a) set" + "; ack2wait-set-by-get-of-ApplyResult4Xi to: " + str(ack2wait)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #PX: True
                     
            ApplyResult4Xi.ack4manoeuvres = ApplyResult4Xi.ack4manoeuvres + 1     

        except:
            message2print = "{(Step 3/6z) yielded" + "; nack2wait-yielded-by-get-of-ApplyResult4Xi: " + str(nack2wait)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            ApplyResult4Xi.nack4manoeuvres = ApplyResult4Xi.nack4manoeuvres + 1
            
        # -----------------------------------------------------------------------------------------    
        # (Step 4/6) 
        # -----------------------------------------------------------------------------------------              
        ack2ready = False
        nack2ready = True
        
        if not self.ready():
            message2print = "{(Step 4/6a) yielded" + "; nack2ready-yielded-by-get-of-ApplyResult4Xi: " + str(nack2ready)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #PX: True
                           
        else:
            ack2ready = True
            
            message2print = "{(Step 4/6b) set" + "; ack2ready-set-by-get-of-ApplyResult4Xi to: " + str(ack2ready)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            ApplyResult4Xi.ack4manoeuvres = ApplyResult4Xi.ack4manoeuvres + 1 
                         
        # -----------------------------------------------------------------------------------------    
        # (Step 5/6) 
        # -----------------------------------------------------------------------------------------              
        try:       
            if self._success:
                message2print = "{(Step 5/6a) returning" + "; value-of-applyresult2xi-returning-by-get-of-ApplyResult4Xi: " + str(self._value) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)            
                #PX: 
             
                value2return = self._value
                
                ApplyResult4Xi.ack4manoeuvres = ApplyResult4Xi.ack4manoeuvres + 1 
                
            else:
                value2return = 0
                
                message2print = "{(Step 5/6z1) warning" + "; ** value2return-of-applyresult2xi-warning-by-get-of-ApplyResult4Xi: "  + str(value2return) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #PX: 
                         
                ApplyResult4Xi.nack4manoeuvres = ApplyResult4Xi.nack4manoeuvres + 1
        except:
                value2return = 0
                
                message2print = "{(Step 5/6z2) failed" + "; _success-of-applyresult2xi-failed-by-get-of-ApplyResult4Xi: "  + str(None) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                         
                ApplyResult4Xi.nack4manoeuvres = ApplyResult4Xi.nack4manoeuvres + 1            
        # -----------------------------------------------------------------------------------------    
        # (Step 6/6) 
        # -----------------------------------------------------------------------------------------                   
        message2print = "{(Step 6/6) returning" + "; value2return-returning-by-get-of-ApplyResult4Xi: " + str(value2return) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        #  
                                         
        
        return value2return

    #</methodize4xi> 
    
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set-of-ApplyResult4Xi" usage = "sets success and value">
    # ---------------------------------------------------------------------------------------------
    def _set(self, i, obj):
        '''
        sets success and value
        '''
                      
        # -----------------------------------------------------------------------------------------    
        # (Step 1/7)
        # -----------------------------------------------------------------------------------------        
           
        class2print = 'ApplyResult4Xi'
        method2print = '_set' 
   
        
        message2print = "{(Step 1/7a) started" + "; set-of-ApplyResult4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
      
        message2print = "{(Step 1/7b) passed" + "; index-passed-to-set-of-ApplyResult4Xi: " + str(i) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
    
        message2print = "{(Step 1/7c) passed" + "; object-passed-to-set-of-ApplyResult4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                      
        # -----------------------------------------------------------------------------------------    
        # (Step 2/7)
        # -----------------------------------------------------------------------------------------        
        self._success, self._value = obj
        
        message2print = "{(Step 2/7a) set" + "; success-of-applyresult2xi-set-by-set-of-ApplyResult4Xi to: " + str(self._success)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
           
        message2print = "{(Step 2/7b) set" + "; value-of-applyresult2xi-set-by-set-of-ApplyResult4Xi to: " + str(self._value)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                
        # -----------------------------------------------------------------------------------------    
        # (Step 3/7)
        # -----------------------------------------------------------------------------------------                        
        if self._callback and self._success:
            self._callback(self._value)
        if self._error_callback and not self._success:
            self._error_callback(self._value)
        self._event.set()
        del self._cache[self._job]
        
                
        # -----------------------------------------------------------------------------------------    
        # (Step 7/7)
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; set-of-ApplyResult4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)       
   
    #</methodize4xi> 
         

                                       
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-ApplyResult4Xi" usage = "senses ApplyResult4Xi class objects">
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        senses ApplyResult4Xi class objects
        '''
          
        
        class2print = 'ApplyResult4Xi'
        method2print = 'sense'
        
        
        message2print = "{started" + "; sense-of-ApplyResult4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
       
  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; ApplyResult4Xi-sensing-by-sense-of-ApplyResult4Xi: " + str(ApplyResult4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-ApplyResult4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['inheritance-of-applyresult2xi'] = object    
    
            kb2xi['ack4manoeuvres-of-applyresult2xi'] = ApplyResult4Xi.ack4manoeuvres
            kb2xi['nack4manoeuvres-of-applyresult2xi'] = ApplyResult4Xi.nack4manoeuvres
                

        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-ApplyResult4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 

            
        message2print = "{sensed" + "; ApplyResult4Xi-sensed-by-sense-of-ApplyResult4Xi: " + str(ApplyResult4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-ApplyResult4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)             
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi> 
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-ApplyResult4Xi" usage = "reasons ApplyResult4Xi class objects">
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        reasons ApplyResult4Xi class objects
        '''
                 
        
        class2print = 'ApplyResult4Xi'
        method2print = 'reason'
        
        
        message2print = "{started" + "; reason-of-ApplyResult4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
       
   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; ApplyResult4Xi-reasoning-by-reason-of-ApplyResult4Xi: "  + str(ApplyResult4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-ApplyResult4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{reasoned" + "; ApplyResult4Xi-reasoned-by-reason-of-ApplyResult4Xi: "  + str(ApplyResult4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-ApplyResult4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)             
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi> 
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-ApplyResult4Xi" usage = "aces ApplyResult4Xi class objects">
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        aces ApplyResult4Xi class objects 
        '''
               
        
        
        class2print = 'ApplyResult4Xi'
        method2print = 'ace'
        
        
        message2print = "{started" + "; ace-of-ApplyResult4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
         
           
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; ApplyResult4Xi-acing-by-ace-of-ApplyResult4Xi: " + "ApplyResult4Xi" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-ApplyResult4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; ApplyResult4Xi-aced-by-ace-of-ApplyResult4Xi: " + "ApplyResult4Xi" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-ApplyResult4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)             
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
    #</methodize4xi> 
#</class4xi>
#</program4xi>


# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_proxy_mp_xi4.txt
#   sense-proxy-mp-test4xi.xml
#   reason-proxy-mp-test4xi.xml
#   ace-proxy-mp-test4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of applyresult4xi.py                            
# ------------------------------------------------------------------------------------------------- 
#"
#/>
 